@@ -11,11 +11,9 @@ const Main = imports.ui.main;
 const Params = imports.misc.params;
 const PopupMenu = imports.ui.popupMenu;
 
-function ButtonBox(params) {
-    this._init.apply(this, arguments);
-};
+const ButtonBox = new Lang.Class({
+    Name: 'ButtonBox',
 
-ButtonBox.prototype = {
     _init: function(params) {
         params = Params.parse(params, { style_class: 'panel-button' }, true);
         this.actor = new Shell.GenericContainer(params);
@@ -92,31 +90,45 @@ ButtonBox.prototype = {
 
         child.allocate(childBox, flags);
     },
-}
+});
 
-function Button(menuAlignment) {
-    this._init(menuAlignment);
-}
+const Button = new Lang.Class({
+    Name: 'PanelMenuButton',
+    Extends: ButtonBox,
 
-Button.prototype = {
-    __proto__: ButtonBox.prototype,
-
-    _init: function(menuAlignment) {
-        ButtonBox.prototype._init.call(this, { reactive: true,
-                                               can_focus: true,
-                                               track_hover: true });
+    _init: function(menuAlignment, dontCreateMenu) {
+        this.parent({ reactive: true,
+                      can_focus: true,
+                      track_hover: true });
 
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
         this.actor.connect('key-press-event', Lang.bind(this, this._onSourceKeyPress));
-        this.menu = new PopupMenu.PopupMenu(this.actor, menuAlignment, St.Side.TOP);
-        this.menu.actor.add_style_class_name('panel-menu');
-        this.menu.connect('open-state-changed', Lang.bind(this, this._onOpenStateChanged));
-        this.menu.actor.connect('key-press-event', Lang.bind(this, this._onMenuKeyPress));
-        Main.uiGroup.add_actor(this.menu.actor);
-        this.menu.actor.hide();
+
+        if (dontCreateMenu)
+            this.menu = null;
+        else
+            this.setMenu(new PopupMenu.PopupMenu(this.actor, menuAlignment, St.Side.TOP, 0));
+    },
+
+    setMenu: function(menu) {
+        if (this.menu)
+            this.menu.destroy();
+
+        this.menu = menu;
+        if (this.menu) {
+            this.menu.actor.add_style_class_name('panel-menu');
+            this.menu.connect('open-state-changed', Lang.bind(this, this._onOpenStateChanged));
+            this.menu.actor.connect('key-press-event', Lang.bind(this, this._onMenuKeyPress));
+
+            Main.uiGroup.add_actor(this.menu.actor);
+            this.menu.actor.hide();
+        }
     },
 
     _onButtonPress: function(actor, event) {
+        if (!this.menu)
+            return;
+
         if (!this.menu.isOpen) {
             // Setting the max-height won't do any good if the minimum height of the
             // menu is higher then the screen; it's useful if part of the menu is
@@ -130,6 +142,9 @@ Button.prototype = {
     },
 
     _onSourceKeyPress: function(actor, event) {
+        if (!this.menu)
+            return false;
+
         let symbol = event.get_key_symbol();
         if (symbol == Clutter.KEY_space || symbol == Clutter.KEY_Return) {
             this.menu.toggle();
@@ -175,7 +190,7 @@ Button.prototype = {
 
         this.emit('destroy');
     }
-};
+});
 Signals.addSignalMethods(Button.prototype);
 
 /* SystemStatusButton:
@@ -184,15 +199,13 @@ Signals.addSignalMethods(Button.prototype);
  * volume, bluetooth...), which is just a PanelMenuButton with an
  * icon and a tooltip
  */
-function SystemStatusButton() {
-    this._init.apply(this, arguments);
-}
-
-SystemStatusButton.prototype = {
-    __proto__: Button.prototype,
+const SystemStatusButton = new Lang.Class({
+    Name: 'SystemStatusButton',
+    Extends: Button,
 
     _init: function(iconName,tooltipText) {
-        Button.prototype._init.call(this, 0.0);
+        this.parent(0.0);
+
         this._iconActor = new St.Icon({ icon_name: iconName,
                                         icon_type: St.IconType.SYMBOLIC,
                                         style_class: 'system-status-icon' });
@@ -219,4 +232,4 @@ SystemStatusButton.prototype = {
             this.tooltip = null;
         }
     }
-};
+});
