@@ -239,7 +239,8 @@ const IMStatusChooserItem = new Lang.Class({
     },
 
     _setIconFromFile: function(iconFile) {
-        this._iconBin.set_style('background-image: url("' + iconFile + '");');
+        this._iconBin.set_style('background-image: url("' + iconFile + '");' +
+                                'background-size: contain;');
         this._iconBin.child = null;
     },
 
