@@ -1306,6 +1306,8 @@ st_theme_node_render_resources (StThemeNode   *node,
   StShadow *background_image_shadow_spec;
   const char *background_image;
 
+  g_return_if_fail (width > 0 && height > 0);
+
   texture_cache = st_texture_cache_get_default ();
 
   /* FIXME - need to separate this into things that need to be recomputed on
@@ -1409,9 +1411,11 @@ st_theme_node_render_resources (StThemeNode   *node,
       else if (node->background_color.alpha > 0 || has_border)
         {
           CoglHandle buffer, offscreen;
+          int texture_width = ceil (width);
+          int texture_height = ceil (height);
 
-          buffer = cogl_texture_new_with_size (width,
-                                               height,
+          buffer = cogl_texture_new_with_size (texture_width,
+                                               texture_height,
                                                COGL_TEXTURE_NO_SLICING,
                                                COGL_PIXEL_FORMAT_ANY);
           offscreen = cogl_offscreen_new_to_texture (buffer);
@@ -1419,9 +1423,14 @@ st_theme_node_render_resources (StThemeNode   *node,
           if (offscreen != COGL_INVALID_HANDLE)
             {
               ClutterActorBox box = { 0, 0, width, height };
+              CoglColor clear_color;
 
               cogl_push_framebuffer (offscreen);
               cogl_ortho (0, width, height, 0, 0, 1.0);
+
+              cogl_color_set_from_4ub (&clear_color, 0, 0, 0, 0);
+              cogl_clear (&clear_color, COGL_BUFFER_BIT_COLOR);
+
               st_theme_node_paint_borders (node, &box, 0xFF);
               cogl_pop_framebuffer ();
               cogl_handle_unref (offscreen);
@@ -1920,6 +1929,9 @@ st_theme_node_paint (StThemeNode           *node,
   allocation.x2 = width;
   allocation.y2 = height;
 
+  if (width <= 0 || height <= 0)
+    return;
+
   if (node->alloc_width != width || node->alloc_height != height)
     st_theme_node_render_resources (node, width, height);
 
